const fs = require("fs");
const os = require("os");
const path = require("path");
const ssbKeys = require("ssb-keys");
const serveBlobs = require("./serve-blobs");
const mkdirp = require("mkdirp");

const ssbPath = path.resolve(os.homedir(), '.ssb');
if (!fs.existsSync(ssbPath)) {
  mkdirp.sync(ssbPath);
}
const secretPath = path.join(ssbPath, "/secret");

const keys = (() => {
  if (fs.existsSync(secretPath)) {
    const fileContents = fs.readFileSync(secretPath, "ascii");
    return JSON.parse(fileContents);
  } else {
    const generatedKeys = ssbKeys.generate();
    const fileContents = JSON.stringify(generatedKeys, null, 2);
    fs.writeFileSync(secretPath, fileContents, "ascii");
    return generatedKeys;
  }
})();

const config = require("ssb-config/inject")();

const manifest = {
  auth: "async",
  address: "sync",
  manifest: "sync",
  get: "async",
  createFeedStream: "source",
  createLogStream: "source",
  messagesByType: "source",
  createHistoryStream: "source",
  createUserStream: "source",
  links: "source",
  relatedMessages: "async",
  add: "async",
  publish: "async",
  getAddress: "sync",
  getLatest: "async",
  latest: "source",
  latestSequence: "async",
  whoami: "sync",
  usage: "sync",
  plugins: {
    install: "source",
    uninstall: "source",
    enable: "async",
    disable: "async"
  },
  gossip: {
    peers: "sync",
    add: "sync",
    remove: "sync",
    ping: "duplex",
    connect: "async",
    changes: "source",
    reconnect: "sync"
  },
  friends: {
    get: "async",
    createFriendStream: "source",
    hops: "async"
  },
  replicate: {
    changes: "source",
    upto: "source"
  },
  blobs: {
    get: "source",
    getSlice: "source",
    add: "sink",
    rm: "async",
    ls: "source",
    has: "async",
    size: "async",
    meta: "async",
    want: "async",
    push: "async",
    changes: "source",
    createWants: "source"
  },
  backlinks: {
    read: "source"
  },
  about: {
    stream: "source",
    get: "async"
  },
  contacts: {
    stream: "source",
    get: "async"
  },
  query: {
    read: "source"
  },
  invite: {
    create: "async",
    accept: "async",
    use: "async"
  },
  block: {
    isBlocked: "sync"
  },
  private: {
    publish: "async",
    unbox: "sync",
    read: "source"
  }
};

const createSbot = require("scuttlebot/index")
  .use(require("scuttlebot/plugins/plugins"))
  .use(require("scuttlebot/plugins/master"))
  .use(require("scuttlebot/plugins/gossip"))
  .use(require("scuttlebot/plugins/replicate"))
  .use(require("ssb-friends"))
  .use(require("ssb-blobs"))
  .use(require("ssb-backlinks"))
  .use(require("ssb-private"))
  .use(require("ssb-about"))
  .use(require("ssb-contacts"))
  .use(require("ssb-query"))
  .use(require("scuttlebot/plugins/invite"))
  .use(require("scuttlebot/plugins/block"))
  .use(require("scuttlebot/plugins/local"))
  .use(require("scuttlebot/plugins/logging"));

// add third-party plugins
require("scuttlebot/plugins/plugins").loadUserPlugins(createSbot, config);

// start server
config.keys = keys;
config.path = ssbPath;
const sbot = createSbot(config);
serveBlobs(sbot);
